@@ -6,6 +6,7 @@ from scipy.optimize import minimize
 from .modified_optimize import _minimize_powell
 from .fit_tools import *
 from .analyse_window import sidelob_param
+from .save_chain import save_sampled_chain
 from pathos.multiprocessing import ProcessPool
 import sys
 import numdifftools as nd
@@ -15,6 +16,8 @@ import matplotlib.pyplot as plt
 import emcee
 import corner
 from apollinaire.psd import degrade_psd
+from scipy.stats import linregress
+import warnings
 
 '''
 This file is constituded with MLE tools used for fitting stellar 
@@ -40,42 +43,47 @@ def create_labels (n_harvey, frozen_param) :
 
   label = np.array (label)
   label = label[~frozen_param]
-
+ 
   return label
 
-def get_low_bounds (full_param, n_harvey, freq, psd) :
+def get_low_bounds (full_param, n_harvey, freq, psd, numax) :
   '''
   Low bound for background parameters. 
   '''
-  low_bounds = 0.1 * full_param
+  low_bounds = np.full (full_param.size, 1e-15)
   for ii in range (n_harvey) :
-    if ii==1 :
-      low_bounds[3*ii+1] = 1.e-8 #constraint on Harvey law cut frequency
+    if ii==0 :
+      low_bounds[3*ii+1] = 0.1 * numax #constraint on Harvey law cut frequency
+      if full_param[3*ii+1] < low_bounds[3*ii+1] : #Case when spectro is True
+        low_bounds[3*ii+1] = freq[0] 
     else :
-      low_bounds[3*ii+1] = 0.5 * full_param[3*ii+1]
+      low_bounds[3*ii+1] = min (0.7 * numax, 0.8 *full_param[3*ii+1])
     low_bounds[3*ii+2] = 1. #constraint on Harvey law exponent
-  low_bounds[-2:-5:-1] = 0.5 * full_param[-2:-5:-1] #constraint on gaussian envelope
+  low_bounds[-4] = 1e-15
+  low_bounds[-3] = 0.5 * numax 
+  low_bounds[-2] = 0.05 * numax
 
   return low_bounds
 
-def get_up_bounds (full_param, n_harvey, freq, psd) :
+def get_up_bounds (full_param, n_harvey, freq, psd, numax) :
   '''
   Up bound for background parameters. 
   '''
 
   h = freq[2] - freq[1]
-  rms = np.sqrt (np.sum (psd) * h)
+  square_rms = np.sum (psd) * h
 
   up_bounds = 10. * full_param
   for ii in range (n_harvey) :
-    if ii > 1 :
-      up_bounds[3*ii] = 0.5 * rms 
-      up_bounds[3*ii+1] = freq[-1] #constraint on Harvey law cut frequency
-    elif n_harvey > 1 :
-      up_bounds[3*ii] = 2 * rms 
-      up_bounds[3*ii+1] = 1.5 * full_param[3*ii+1] #constraint on Harvey law cut frequency
+    up_bounds[3*ii] = square_rms * 1e6 
+    if ii==0 :
+      up_bounds[3*ii+1] = 0.7 * numax #constraint on Harvey law cut frequency
+    else :
+      up_bounds[3*ii+1] = 2. * numax #constraint on Harvey law cut frequency
     up_bounds[3*ii+2] = 5. #constraint on Harvey law exponent 
-  up_bounds[-2:-5:-1] = 2. * full_param[-2:-5:-1]  #constraint on gaussian envelope
+  up_bounds[-4] = 0.5 * square_rms * 1e6
+  up_bounds[-3] = 1.5 * numax 
+  up_bounds[-2] = 0.5 * numax
 
   return up_bounds
 
@@ -102,7 +110,23 @@ def dnu_scale (r, m) :
 
   return dnu
 
-def background_guess (freq, psd, numax, dnu, m_star=None, n_harvey=2, spectro=False) :
+def power_law_guess (freq, psd, high_cut=20.) :
+
+  '''
+  Build power law guess.
+  '''
+
+  log_psd = np.log (psd[freq<high_cut])
+  log_freq = np.log (freq[freq<high_cut])
+  
+  minusb, lna, rvalue, pvalue, stderr = linregress (log_freq, log_psd) 
+  b = - minusb
+  a = np.exp (lna)
+
+  return a, b
+
+def background_guess (freq, psd, numax, dnu, m_star=None, n_harvey=2, spectro=False, power_law=False,
+                      high_cut_plaw=20.) :
   '''
   Create a 1D vector with the initial guess to feed the MLE minimisation function.
   Scaling laws for nu_c and A are taken from Kallinger et al. 2014. 
@@ -117,6 +141,10 @@ def background_guess (freq, psd, numax, dnu, m_star=None, n_harvey=2, spectro=Fa
     default ``False``.
   :type spectro: bool
 
+  :param high_cut_plaw: high cut in frequency to compute the power law guess. Optional,
+    default 20. 
+  :type high_cut_plaw: float
+
   :return: 1D vector of background initial parameter.
   :rtype: ndarray
   '''
@@ -169,21 +197,28 @@ def background_guess (freq, psd, numax, dnu, m_star=None, n_harvey=2, spectro=Fa
       guess[8+ii*3] = 4
 
   # Power law
-  #TODO
-  print ('Power law guess need to be added to the function, this guess will build a null power law vector')
-  guess[n_harvey*3] = 0
-  guess[n_harvey*3+1] = -1
+  if power_law :
+    a, b = power_law_guess (freq, psd, high_cut=high_cut_plaw)
+    if b < 0 :
+      a = 1.e-15
+      b = 1.e-15 #b cannot be less than 0. If fit returned b value < 0, fix the initial guess to minimum value.  
+    guess[n_harvey*3] = a
+    guess[n_harvey*3+1] = b
+  else :
+    guess[n_harvey*3] = 0
+    guess[n_harvey*3+1] = -1
 
   # Gaussian parameters of the envelope
   # Hmax
-  Hmax = np.mean (psd[(freq>numax-dnu/4.)&(freq<numax+dnu/4.)])
+  Wenv = 0.175 * numax
+  Hmax = 1.5 * np.mean (psd[(freq>numax-Wenv)&(freq<numax+Wenv)])
   if np.isnan (Hmax) :
-    Hmax = 0
+    Hmax = 1e-12
   guess[n_harvey*3+2] = Hmax #to adjust if needed 
   # numax
   guess[n_harvey*3+3] = numax 
   # Wenv
-  guess[n_harvey*3+4] = 4 * dnu #to adjust if needed 
+  guess[n_harvey*3+4] = Wenv #to adjust if needed 
 
   # white noise estimation with high frequency region. 
   nu_nyq = freq[-1]
@@ -208,7 +243,7 @@ def power_law (freq, a, b) :
   '''
   Compute power law.
   ''' 
-  return np.power (a*freq, -b)
+  return a * np.power (freq, -b)
 
 def gauss_env (freq, Hmax, numax, Wenv) :
   '''
@@ -217,12 +252,12 @@ def gauss_env (freq, Hmax, numax, Wenv) :
   return Hmax * np.exp (- np.power ((freq-numax)/Wenv, 2))
 
 def background_model (freq, param_harvey=None, param_plaw=None, 
-                      param_gaussian=None, noise=0, n_harvey=2) :
+                      param_gaussian=None, noise=0, n_harvey=2, apodisation=False) :
   '''
   Compute background model.
   '''
   model = np.zeros (freq.size)
-  if param_harvey is not None :
+  if param_harvey is not None and n_harvey>0 :
     param_harvey = np.reshape (param_harvey, (n_harvey, param_harvey.size//n_harvey))
     for elt in param_harvey :
       model += harvey (freq, *elt) 
@@ -230,6 +265,10 @@ def background_model (freq, param_harvey=None, param_plaw=None,
     model += power_law (freq, *param_plaw) 
   if param_gaussian is not None :
     model += gauss_env (freq, *param_gaussian) 
+  if apodisation :
+    nyquist = freq[-1]
+    eta = np.sinc (np.pi / 2 * freq / nyquist) 
+    model = model * eta * eta
   model += noise
 
   return model
@@ -259,6 +298,39 @@ def extract_param (param, n_harvey) :
 
   return param_harvey, param_plaw, param_gaussian, white_noise
 
+def build_background (freq, param, n_harvey=2, apodisation=False,
+                      remove_gaussian=True) :
+  '''
+  Rebuild background vector from complete set of parameters.
+
+  :param freq: frequency vector (in muHz).
+  :type freq: ndarray
+
+  :type param: 1D vector with backgrounds parameters given in the following order:
+    param_harvey (3x``n_harvey``), param_plaw (2), param_gaussian (3), white noise (1)
+  :type param: array like
+
+  :param n_harvey: number of Harvey laws to use to build the background
+    model. Optional, default 2.  
+  :type n_harvey: int
+
+  :param apodisation: if set to ``True``, distort the model to take the 
+    apodisation into account. Optional, default ``False``.
+  :type apodisation: bool
+
+  :param remove_gaussian: if set to ``True``, the p-mode gaussian bump will not be 
+    taken into account when building the vector. Optional, default ``True``. 
+  :param remove_gaussian: bool
+  '''
+
+  param_harvey, param_plaw, param_gaussian, noise = extract_param (param, n_harvey)
+  if remove_gaussian :
+    param_gaussian=None
+  back = background_model (freq, param_harvey, param_plaw, param_gaussian=param_gaussian, 
+                           noise=noise, n_harvey=n_harvey, apodisation=apodisation)
+
+  return back
+
 def log_prior (param, bounds) :
   '''
   Compute positive log_prior probability for MCMC framework. Uninformative priors are used.  
@@ -285,7 +357,8 @@ def log_prior (param, bounds) :
 
   return l_prior
 
-def log_likelihood_back (input_param, freq, psd, full_param, frozen_param, n_harvey, fit_log=False) :
+def log_likelihood_back (input_param, freq, psd, full_param, frozen_param, n_harvey, fit_log=False, 
+                         apodisation=False) :
   '''
   Compute negative log_likelihood for fitting model on 
   background.
@@ -324,7 +397,7 @@ def log_likelihood_back (input_param, freq, psd, full_param, frozen_param, n_har
 
   full_param[~frozen_param] = param
   param_harvey, param_plaw, param_gaussian, noise = extract_param (full_param, n_harvey)
-  model = background_model (freq, param_harvey, param_plaw, param_gaussian, noise, n_harvey)
+  model = background_model (freq, param_harvey, param_plaw, param_gaussian, noise, n_harvey, apodisation=apodisation)
 
   aux = psd / model + np.log (model)
   log_l = np.sum (aux)
@@ -396,17 +469,17 @@ def log_probability_back (param_to_fit, freq, psd, bounds, full_param, frozen_pa
   return l_proba
 
 def visualise_background (freq, psd, param_fitted=None, guess=None, low_cut=100., n_harvey=2, 
-                          filename=None, spectro=True, alpha=1., show=False) :
+                          filename=None, spectro=True, alpha=1., show=False, apodisation=False) :
   '''
   Plot fitted background against real PSD (and possibly against initial guess).
   '''
 
   if guess is not None :
     guess_harvey, guess_plaw, guess_gaussian, guess_noise = extract_param (guess, n_harvey)
-    guess_model = background_model (freq, guess_harvey, guess_plaw, guess_gaussian, guess_noise, n_harvey)
+    guess_model = background_model (freq, guess_harvey, guess_plaw, guess_gaussian, guess_noise, n_harvey, apodisation=apodisation)
   if param_fitted is not None :
     fitted_harvey, fitted_plaw, fitted_gaussian, fitted_noise = extract_param (param_fitted, n_harvey)
-    fitted_back = background_model (freq, fitted_harvey, fitted_plaw, fitted_gaussian, fitted_noise, n_harvey)
+    fitted_back = background_model (freq, fitted_harvey, fitted_plaw, fitted_gaussian, fitted_noise, n_harvey, apodisation=apodisation)
 
   fig = plt.figure (figsize=(12,6))
   ax = fig.add_subplot (111)
@@ -415,16 +488,24 @@ def visualise_background (freq, psd, param_fitted=None, guess=None, low_cut=100.
   ax.plot (freq[freq>low_cut], psd[freq>low_cut], color='black')
   if guess is not None :
     ax.plot (freq, guess_model, color='green')
-    guess_harvey = np.reshape (guess_harvey, (n_harvey, guess_harvey.size//n_harvey))
-    for elt in guess_harvey :
-      ax.plot (freq, harvey (freq, *elt), ':', color='green') 
-    ax.plot (freq, gauss_env (freq, *guess_gaussian), ':', color='green')
+    if n_harvey > 0 :
+      guess_harvey = np.reshape (guess_harvey, (n_harvey, guess_harvey.size//n_harvey))
+      for elt in guess_harvey :
+        ax.plot (freq, harvey (freq, *elt), ':', color='green') 
+    if guess_gaussian[0] > 0 :
+      ax.plot (freq, gauss_env (freq, *guess_gaussian), ':', color='green')
+    if guess_plaw[0]!= 0 :
+      ax.plot (freq, power_law (freq, *guess_plaw), ':', color='green')
   if param_fitted is not None :
     ax.plot (freq, fitted_back, color='red', alpha=alpha)
-    fitted_harvey = np.reshape (fitted_harvey, (n_harvey, fitted_harvey.size//n_harvey))
-    for elt in fitted_harvey :
-      ax.plot (freq, harvey (freq, *elt), '--', color='red', alpha=alpha) 
-    ax.plot (freq, gauss_env (freq, *fitted_gaussian), '--', color='red')
+    if n_harvey > 0 :
+      fitted_harvey = np.reshape (fitted_harvey, (n_harvey, fitted_harvey.size//n_harvey))
+      for elt in fitted_harvey :
+        ax.plot (freq, harvey (freq, *elt), '--', color='red', alpha=alpha) 
+    if fitted_gaussian[0] > 0 :
+      ax.plot (freq, gauss_env (freq, *fitted_gaussian), '--', color='red')
+    if fitted_plaw[0]!= 0 :
+      ax.plot (freq, power_law (freq, *fitted_plaw), '--', color='red')
 
   ax.set_xlabel (r'Frequency ($\mu$Hz)')
   if spectro :
@@ -447,10 +528,12 @@ def visualise_background (freq, psd, param_fitted=None, guess=None, low_cut=100.
   
   return
 
-def perform_mle_background (freq, psd, n_harvey=2, r=1., m=1., teff=5770., guess=None, frozen_param=None, power_law=False,
+def perform_mle_background (freq, psd, n_harvey=2, r=1., m=1., teff=5770., dnu=None, numax=None, 
+                            guess=None, frozen_param=None, power_law=False, gaussian=True, 
                             frozen_harvey_exponent=False, low_cut=100., method=_minimize_powell, fit_log=False,
                             low_bounds=None, up_bounds=None, no_bounds=False, show=True, show_guess=False, filename=None, spectro=True,
-                            quickfit=False, num=5000, smoothing=50) :
+                            quickfit=False, num=5000, reboxing_behaviour='advanced_reboxing', reboxing_strategy='median', 
+                            apodisation=False, high_cut_plaw=20.) :
   '''
   Perform MLE over background model. 
 
@@ -474,10 +557,18 @@ def perform_mle_background (freq, psd, n_harvey=2, r=1., m=1., teff=5770., guess
   :param teff: stellar effective temperature. Given in K. Optional, default 5770. 
   :type teff: float
 
+  :param dnu: large separation of the p-mode. If given, it will superseed the scaling law guess
+    using ``r``, ``m`` and ``teff``. 
+  :type dnu: float
+
+  :param numax: maximum p-mode bump power frequency. If given, it will superseed the scaling law guess
+    using ``r``, ``m`` and ``teff``. 
+  :type numax: float
+
   :param guess: first guess directly passed by the users. If guess is ``None``, the 
     function will automatically infer a first guess. Optional, default ``None``.
     Backgrounds parameters given in the following order:
-    *param_harvey (3 n_harvey), param_plaw (2), param_gaussian (3), white noise (1)*
+    *param_harvey (3* ``n_harvey`` *), param_plaw (2), param_gaussian (3), white noise (1)*
   :type guess: array-like.
 
   :param frozen_param: boolean array of the same size as guess. Components set to ``True``
@@ -488,6 +579,10 @@ def perform_mle_background (freq, psd, n_harvey=2, r=1., m=1., teff=5770., guess
     ``False``.
   :type power_law: bool
 
+  :param gaussian: set to ``True`` to fit the p-mode gaussian on the background. Optional, default 
+    ``True``.
+  :type gaussian: bool
+
   :param frozen_harvey_exponent: set to ``True`` to freeze and not fit Harvey laws exponent to
     4. Optional, default ``False``. 
   :type frozen_harvey_exponent: bool 
@@ -536,22 +631,37 @@ def perform_mle_background (freq, psd, n_harvey=2, r=1., m=1., teff=5770., guess
   :param num: number of points of the logarithmically sampled spectrum if ``quickfit`` is set to ``True``. Optional, default 50.
   :type num: int
 
-  :param smoothing: size of the boxcar window used for the smoothing if set to ``True``. Optional, default 50.
-  :type smoothing: int
+  :param reboxing_behaviour: behaviour for ``quickfit`` new power vector computation. It can be ``smoothing``, ``reboxing`` or 
+    ``advanced_reboxing``. Optional, default ``advanced_reboxing``. 
+  :type reboxing_behaviour: str
+
+  :param reboxing_strategy: reboxing strategy to take box values when using ``quickfit`` and ``reboxing_behaviour=reboxing``. 
+    Can be ``median`` or ``mean``. Optional, default ``median``.
+  :type reboxing_strategy: str
+
+  :param apodisation: if set to ``True``, distort the model to take the apodisation into account. Optional, default ``False``.
+  :type apodisation: bool
+
+  :param high_cut_plaw: high cut in frequency to compute the power law guess. Optional,
+    default 20. 
+  :type high_cut_plaw: float
 
   :return: fitted background and fitted background parameters.  
   :rtype: tuple of array
   '''
 
-  dnu = dnu_scale (r, m)
-  numax = numax_scale (r, m, teff)
-  print ('Dnu computed with scale law:', dnu) 
-  print ('numax computed with scale law:', numax)
+  if dnu is None :
+    dnu = dnu_scale (r, m)
+    print ('Dnu computed with scale law: {:.1f} muHz'.format (dnu)) 
+  if numax is None :
+    numax = numax_scale (r, m, teff)
+    print ('numax computed with scale law: {:.1f} muHz'.format (numax)) 
 
   lenguess = n_harvey * 3 + 6
 
   if guess is None :
-    guess = background_guess (freq, psd, numax, dnu, m_star=m, n_harvey=n_harvey, spectro=spectro) 
+    guess = background_guess (freq, psd, numax, dnu, m_star=m, n_harvey=n_harvey, spectro=spectro, power_law=power_law,
+                              high_cut_plaw=high_cut_plaw) 
 
   elif len (guess) != lenguess :
     raise Exception ('guess length is not consistent')
@@ -569,55 +679,56 @@ def perform_mle_background (freq, psd, n_harvey=2, r=1., m=1., teff=5770., guess
     frozen_param[3*n_harvey] = True 
     frozen_param[3*n_harvey+1] = True
 
+  if not gaussian :
+    guess[n_harvey*3+2] = 0
+    frozen_param[n_harvey*3+2:n_harvey*3+5] = True
+
   param = np.copy (guess[~frozen_param])
   full_param = np.copy (guess)
 
   if show_guess :
     visualise_background (freq, psd, guess=guess, low_cut=low_cut, n_harvey=n_harvey, spectro=spectro) 
 
-  #print ('Guess param')
-  #print (full_param)
-
   #Bounds
-  if  no_bounds :
+  if no_bounds :
     bounds = None
     fit_log = True #the only condition considered is thus that we want positive parameters. 
   else :
     if low_bounds is None :
-      low_bounds = get_low_bounds (full_param, n_harvey, freq, psd)
+      low_bounds = get_low_bounds (full_param, n_harvey, freq, psd, numax)
     if up_bounds is None :
-      up_bounds = get_up_bounds (full_param, n_harvey, freq, psd)
+      up_bounds = get_up_bounds (full_param, n_harvey, freq, psd, numax)
 
-    #print ('Low bound for fit')
-    #print (low_bounds)
-    #print ('Up bound for fit')
-    #print (up_bounds)
+  low_bounds = low_bounds[~frozen_param]
+  up_bounds = up_bounds[~frozen_param]
 
-    low_bounds = low_bounds[~frozen_param]
-    up_bounds = up_bounds[~frozen_param]
-
-    if fit_log :
-      low_bounds = np.log (low_bounds)
-      up_bounds = np.log (up_bounds)
-
-    if method is _minimize_powell :
-      bounds = (low_bounds, up_bounds)
-    else :
-      bounds = np.c_[low_bounds, up_bounds]
+  labels = create_labels (n_harvey, frozen_param)
+  if not no_bounds :
+    check_bounds (param, low_bounds, up_bounds, labels=labels)
 
   if fit_log :
+    low_bounds = np.log (low_bounds)
+    up_bounds = np.log (up_bounds)
     param = np.log (param)
 
+  if method is _minimize_powell :
+    bounds = (low_bounds, up_bounds)
+  else :
+    bounds = np.c_[low_bounds, up_bounds]
+
   # Cut under low_cut
   aux_freq = freq[freq>low_cut]
   aux_psd = psd[freq>low_cut]
 
   if quickfit :
-    aux_freq, aux_psd = degrade_psd (aux_freq, aux_psd, num=num, smoothing=smoothing)
-  print ('Beginning fit')
-  result = minimize (log_likelihood_back, param,
-                     args=(aux_freq, aux_psd, full_param, frozen_param, n_harvey, fit_log), bounds=bounds, 
-                     method=method)
+    aux_freq, aux_psd = degrade_psd (aux_freq, aux_psd, num=num, behaviour=reboxing_behaviour,
+                                     strategy=reboxing_strategy, numax=numax)
+  print ('Background model likelihood minimisation:')
+  with warnings.catch_warnings () :
+    warnings.filterwarnings('ignore')
+    result = minimize (log_likelihood_back, param,
+                       args=(aux_freq, aux_psd, full_param, frozen_param, n_harvey, fit_log), bounds=bounds, 
+                       method=method)
 
   print (result.message)
 
@@ -629,7 +740,7 @@ def perform_mle_background (freq, psd, n_harvey=2, r=1., m=1., teff=5770., guess
   full_param[~frozen_param] = param_back
   param_harvey, param_plaw, param_gaussian, noise = extract_param (full_param, n_harvey)
   # We want the background without the gaussian envelope
-  fitted_back = background_model (freq, param_harvey, param_plaw, param_gaussian=None, noise=noise, n_harvey=n_harvey)
+  fitted_back = background_model (freq, param_harvey, param_plaw, param_gaussian=None, noise=noise, n_harvey=n_harvey, apodisation=apodisation)
 
   visualise_background (freq, psd, param_fitted=full_param, guess=guess, low_cut=low_cut, n_harvey=n_harvey, 
                           filename=filename, spectro=spectro, show=show) 
@@ -637,12 +748,15 @@ def perform_mle_background (freq, psd, n_harvey=2, r=1., m=1., teff=5770., guess
 
   return fitted_back, full_param
 
-def explore_distribution_background (freq, psd,  
-                                     n_harvey=2, r=1., m=1., teff=5770., guess=None, frozen_param=None, power_law=False,
+def explore_distribution_background (freq, psd, n_harvey=2, r=1., m=1., teff=5770., dnu=None, numax=None, 
+                                     guess=None, frozen_param=None, power_law=False, gaussian=True, 
                                      frozen_harvey_exponent=False, low_cut=100., fit_log=False,
                                      low_bounds=None, up_bounds=None, spectro=True, show=True, show_guess=False, show_corner=True,
                                      nsteps=1000, filename=None, parallelise=False, progress=False, nwalkers=64, filemcmc=None,
-                                     coeff_discard=50, thin=1, quickfit=False, num=5000, smoothing=50) :
+                                     coeff_discard=50, thin=1, quickfit=False, num=5000, reboxing_behaviour='advanced_reboxing', 
+                                     reboxing_strategy='median',  
+                                     save_only_after_sampling=False, apodisation=False, high_cut_plaw=20., bins=100,
+                                     existing_chains='read') :
 
   '''
   Use a MCMC framework to fit the background.  
@@ -667,10 +781,18 @@ def explore_distribution_background (freq, psd,
   :param teff: stellar effective temperature. Given in K. Optional, default 5770. 
   :type teff: float
 
+  :param dnu: large separation of the p-mode. If given, it will superseed the scaling law guess
+    using ``r``, ``m`` and ``teff``. 
+  :type dnu: float
+
+  :param numax: maximum p-mode bump power frequency. If given, it will superseed the scaling law guess
+    using ``r``, ``m`` and ``teff``. 
+  :type numax: float
+
   :param guess: first guess directly passed by the user. If guess is ``None``, the 
     function will automatically infer a first guess. Optional, default ``None``.
     Backgrounds parameters given in the following order:
-    *param_harvey (3 ``n_harvey``), param_plaw (2), param_gaussian (3), white noise (1)*
+    *param_harvey (3* ``n_harvey`` *), param_plaw (2), param_gaussian (3), white noise (1)*
   :type guess: array-like.
 
   :param frozen_param: boolean array of the same size as guess. Components set to ``True``
@@ -681,6 +803,10 @@ def explore_distribution_background (freq, psd,
     ``False``.
   :type power_law: bool
 
+  :param gaussian: set to ``True`` to fit the p-mode gaussian on the background. Optional, default 
+    ``True``.
+  :type gaussian: bool
+
   :param frozen_harvey_exponent: set to ``True`` to freeze and not fit Harvey laws exponent to
     4. Optional, default ``False``. 
   :type frozen_harvey_exponent: bool 
@@ -724,6 +850,9 @@ def explore_distribution_background (freq, psd,
     Plot will be saved as a pdf is ``filemcmc`` is also specified. Optional, default ``True``.
   :type show: bool
 
+  :param nsteps: number of MCMC steps to perform.
+  :type nsteps: int
+
   :param show_guess: if set to ``True``, will show at the beginning a plot summarising the guess. Optional, default ``False``.
   :type show_guess: bool
 
@@ -741,23 +870,60 @@ def explore_distribution_background (freq, psd,
   :param num: number of points of the logarithmically sampled spectrum if ``quickfit`` is set to ``True``. Optional, default 50.
   :type num: int
 
-  :param smoothing: size of the boxcar window used for the smoothing if set to ``True``. Optional, default 50.
-  :type smoothing: int
+  :param reboxing_behaviour: behaviour for ``quickfit`` new power vector computation. It can be ``smoothing``, ``reboxing`` or 
+    ``advanced_reboxing``. Optional, default ``advanced_reboxing``. 
+  :type reboxing_behaviour: str
+
+  :param reboxing_strategy: reboxing strategy to take box values when using ``quickfit`` and ``reboxing_behaviour=reboxing``. 
+    Can be ``median`` or ``mean``. Optional, default ``median``.
+  :type reboxing_strategy: str
+
+  :param save_only_after_sampling: if set to True, hdf5 file with chains information will only be saved at the end of the sampling
+    process. If set to False, the file will be saved step by step (see ``emcee`` documentation).
+  :type saveon_only_after_sampling: bool
+
+  :param apodisation: if set to ``True``, distort the model to take the apodisation into account. Optional, default ``False``.
+  :type apodisation: bool
+
+  :param high_cut_plaw: high cut in frequency to compute the power law guess. Optional,
+    default 20. 
+  :type high_cut_plaw: float
+
+  :param bins: number of bins for each cornerplot panel. Optional, default 100.
+  :type bins: int
+
+  :param existing_chains: controls the behaviour of the function concerning existing hdf5 files. If, ``read``, existing files will be read
+    without sampling and function output will be updated consequently, if ``reset``, the backend will be cleared and the chain will 
+    be sampled from scratch, if ``sample`` the function will sample the chain from where the previous exploration was stopped.
+    Optional, default ``read``.
+  :type existing_chains: str
 
   :return: the fitted param, its param and their sigma values as determined by the MCMC walk.
   :rtype: tuple of array
   '''
 
-  dnu = dnu_scale (r, m)
-  numax = numax_scale (r, m, teff)
+  if existing_chains=='ignore' :
+    existing_chains = 'read'
+
+  if existing_chains not in ['read', 'reset', 'sample'] :
+    raise Exception ("Unknown value for existing_chains, must be 'read', 'reset' or 'sample'")
+
+  if save_only_after_sampling and existing_chains=='sample' :
+    raise Exception ("save_only_after_sampling=True and existing_chains='sample' are incompatible options.")
+
+  if dnu is None :
+    dnu = dnu_scale (r, m)
+  if numax is None :
+    numax = numax_scale (r, m, teff)
 
   lenguess = n_harvey * 3 + 6
 
   if guess is None :
-    guess = background_guess (freq, psd, numax, dnu, m_star=m, n_harvey=n_harvey, spectro=spectro) 
+    guess = background_guess (freq, psd, numax, dnu, m_star=m, n_harvey=n_harvey, spectro=spectro, power_law=power_law,
+                              high_cut_plaw=high_cut_plaw) 
 
   elif len (guess) != lenguess :
-    raise Exception ('guess length is not consistent')
+    raise Exception ('Guess length is not consistent')
 
   if frozen_param is None :
     frozen_param = np.zeros (guess.size, dtype=bool)
@@ -772,28 +938,27 @@ def explore_distribution_background (freq, psd,
     frozen_param[3*n_harvey] = True 
     frozen_param[3*n_harvey+1] = True
 
+  if not gaussian :
+    guess[n_harvey*3+2] = 0
+    frozen_param[n_harvey*3+2:n_harvey*3+5] = True
+
   param = np.copy (guess[~frozen_param])
   full_param = np.copy (guess)
 
   if show_guess :
     visualise_background (freq, psd, guess=guess, low_cut=low_cut, n_harvey=n_harvey, spectro=spectro) 
 
-  #print ('Guess param')
-  #print (full_param)
-
   #Bounds
   if low_bounds is None :
-    low_bounds = get_low_bounds (full_param, n_harvey, freq, psd)
+    low_bounds = get_low_bounds (full_param, n_harvey, freq, psd, numax)
   if up_bounds is None :
-    up_bounds = get_up_bounds (full_param, n_harvey, freq, psd)
+    up_bounds = get_up_bounds (full_param, n_harvey, freq, psd, numax)
 
-    #print ('Low bound for fit')
-    #print (low_bounds)
-    #print ('Up bound for fit')
-    #print (up_bounds)
+  low_bounds = low_bounds[~frozen_param]
+  up_bounds = up_bounds[~frozen_param]
 
-    low_bounds = low_bounds[~frozen_param]
-    up_bounds = up_bounds[~frozen_param]
+  labels = create_labels (n_harvey, frozen_param)
+  check_bounds (param, low_bounds, up_bounds, labels=labels)
 
   if fit_log :
     low_bounds = np.log (low_bounds)
@@ -806,8 +971,6 @@ def explore_distribution_background (freq, psd,
   aux_freq = freq[freq>low_cut]
   aux_psd = psd[freq>low_cut]
 
-  print ('Beginning fit')
-
   bounds = np.c_[low_bounds, up_bounds]
 
   norm = np.abs (param)
@@ -828,46 +991,57 @@ def explore_distribution_background (freq, psd,
   pos = param_to_pass + 1e-4 * np.random.randn(nwalkers, param_to_pass.size)
   nwalkers, ndim = pos.shape
 
+  run_sampler = True
   if filemcmc is not None :
-    backend = emcee.backends.HDFBackend(filemcmc)
-    backend.reset(nwalkers, ndim) 
+    if path.exists (filemcmc) :
+      if existing_chains=='read' :
+        print (filemcmc + " already exists, existing chains set to 'read', no sampling has been performed, proceeding to next step.")
+        sampler = emcee.backends.HDFBackend(filemcmc, read_only=True)
+        run_sampler = False
+      elif existing_chains=='reset' :
+        os.remove (filemcmc)
+        backend = emcee.backends.HDFBackend(filemcmc)
+        backend.reset(nwalkers, ndim) 
+        print (filemcmc + " already exists, existing chains set to 'reset', file has been deleted and a new file has been created instead.")
+      elif existing_chains=='sample' :
+        backend = emcee.backends.HDFBackend(filemcmc)
+        pos = None
+        print (filemcmc + " already exists, existing chains set to 'sample', sampling will restart from where it stopped.")
+    else :
+      backend = emcee.backends.HDFBackend(filemcmc)
+      backend.reset(nwalkers, ndim) 
+    
     #saving parameters name and normalisation information
     filemcmc_info = filemcmc[:len(filemcmc)-3] + '.dat'
     np.savetxt (filemcmc_info, np.c_[norm], fmt='%-s') 
+
+    if save_only_after_sampling :
+      # I have deliberately created the file to signal to other process that this chain is being
+      # sampled
+      backend = None
   else :
     backend = None
 
-  if quickfit :
-    aux_freq, aux_psd = degrade_psd (aux_freq, aux_psd, num=num, smoothing=smoothing)
+  if run_sampler :
+    if quickfit :
+      aux_freq, aux_psd = degrade_psd (aux_freq, aux_psd, num=num, behaviour=reboxing_behaviour,  
+                                       strategy=reboxing_strategy, numax=numax)
 
-  sampler = emcee.EnsembleSampler(nwalkers, ndim, log_probability_back, 
-                                  args=(aux_freq, aux_psd, bounds_to_pass, full_param, frozen_param, n_harvey, 
-                                  fit_log, norm), 
-                                  backend=backend, pool=pool)
-  sampler.run_mcmc(pos, nsteps, progress=progress)
+    print ('Beginning fit')
+    sampler = emcee.EnsembleSampler(nwalkers, ndim, log_probability_back, 
+                                    args=(aux_freq, aux_psd, bounds_to_pass, full_param, frozen_param, n_harvey, 
+                                    fit_log, norm), 
+                                    backend=backend, pool=pool)
+    sampler.run_mcmc(pos, nsteps, progress=progress)
+
+    if filemcmc is not None :
+      if save_only_after_sampling :
+        save_sampled_chain (filemcmc, sampler, ndim, nwalkers, nsteps)
 
   discard = nsteps // coeff_discard
 
   if show_corner :
-    labels = create_labels (n_harvey, frozen_param)
-    sample_to_plot = sampler.get_chain(discard=discard, thin=thin, flat=True)
-    fig = corner.corner(sample_to_plot*norm, bins=100, labels=labels, quantiles=[0.16,0.84], show_titles=True, title_fmt='.4f')
-    fig.set_size_inches(24,24)
-    axes = np.array(fig.axes).reshape((ndim, ndim))
-    for yi in range(ndim):
-      for xi in range(yi):
-        ax = axes[yi, xi]
-        ax.tick_params (labelsize=6)
-        ax.xaxis.set_major_formatter(ticker.ScalarFormatter(useMathText=True))
-        ax.yaxis.set_major_formatter(ticker.ScalarFormatter(useMathText=True))
-        ax.ticklabel_format (scilimits=(0,0), useOffset=False)
-        if xi!=0 :
-          ax.yaxis.set_ticklabels ([])
-        if yi!=ndim-1 :
-          ax.xaxis.set_ticklabels ([])
-    if filemcmc is not None :
-      plt.savefig (filemcmc[:len(filemcmc[:-3])]+'_cornerplot.pdf', format='pdf') 
-    plt.close ()
+    make_cornerplot (sampler, ndim, discard, thin, labels, norm, filemcmc=filemcmc, bins=bins)
 
   flat_samples = sampler.get_chain(discard=discard, thin=thin, flat=True)
   centiles = np.percentile(flat_samples, [16, 50, 84], axis=0) * norm 
@@ -883,7 +1057,7 @@ def explore_distribution_background (freq, psd,
   full_sigma[~frozen_param] = sigma_back
   param_harvey, param_plaw, param_gaussian, noise = extract_param (full_param, n_harvey)
   # We want the background without the gaussian envelope
-  fitted_back = background_model (freq, param_harvey, param_plaw, param_gaussian=None, noise=noise, n_harvey=n_harvey)
+  fitted_back = background_model (freq, param_harvey, param_plaw, param_gaussian=None, noise=noise, n_harvey=n_harvey, apodisation=apodisation)
 
   visualise_background (freq, psd, param_fitted=full_param, guess=guess, low_cut=low_cut, n_harvey=n_harvey, 
                           filename=filename, spectro=spectro, alpha=0.8, show=show) 
