@@ -3,8 +3,9 @@ import pandas as pd
 import emcee
 import corner
 from .likelihood import log_likelihood, cond_transf
-from .fit_tools import a2z_df_to_param
+from .fit_tools import a2z_df_to_param, a2z_to_pkb, save_a2z
 from .analyse_window import sidelob_param
+from .save_chain import save_sampled_chain
 import matplotlib
 import matplotlib.ticker as ticker
 import matplotlib.pyplot as plt
@@ -12,6 +13,9 @@ import dill
 from pathos.multiprocessing import ProcessPool
 from multiprocessing import Pool
 from os import path
+import glob
+import os
+from os import path
 
 def log_prior (bounds, param=None, param_type=None) :
 
@@ -38,15 +42,20 @@ def log_prior (bounds, param=None, param_type=None) :
   if np.any (cond) :
     return - np.inf
 
-  extent = bounds[:,1] - bounds[:,0] 
-  individual_prior = 1. / extent #assuming uniform law for all given parameters.
+  individual_prior = np.ones (param.size) #assuming uniform law for all given parameters.
+
+  if param_type is not None :
+    angle = param_type=='angle'
+    individual_prior[angle] = np.sin (2*np.pi*param[angle]/360.)
+
   prior = np.prod (individual_prior) 
   l_prior = np.log (prior)
 
   return l_prior
 
 def log_probability (param_to_fit, param_type, freq, psd, back, df_a2z, transform, bounds, param_wdw=None,
-                     norm=None, instr='kepler', use_sinc=False, asym_profile='korzennik') :
+                     norm=None, instr='kepler', use_sinc=False, asym_profile='korzennik', fit_amp=False,
+                     projected_splittings=False) :
   '''
   Compute the positive posterior log probability (unnormalised) of the parameters to fit. 
 
@@ -104,19 +113,23 @@ def log_probability (param_to_fit, param_type, freq, psd, back, df_a2z, transfor
     bounds[:,0] = bounds[:,0] * norm
     bounds[:,1] = bounds[:,1] * norm
 
-  l_prior = log_prior (bounds, param_to_fit)
+  l_prior = log_prior (bounds, param_to_fit, param_type)
 
   if not np.isfinite (l_prior) :
     return - np.inf 
 
   l_likelihood = - log_likelihood (param_to_fit, param_type, freq, psd, back, df_a2z, transform, instr=instr,
-                                   param_wdw=param_wdw, use_sinc=use_sinc, asym_profile=asym_profile)
+                                   param_wdw=param_wdw, use_sinc=use_sinc, asym_profile=asym_profile, fit_amp=fit_amp,
+                                   projected_splittings=projected_splittings)
 
   l_proba = l_prior + l_likelihood
 
   return l_proba
 
 def sort_chain (labels, degrees, flatchain) :
+  '''
+  Sort chains to have fitted parameters in alphabetical order in the corner plot.
+  '''
 
   aux_flat = np.copy (flatchain)
 
@@ -136,7 +149,8 @@ def explore_distribution (result, param_type, freq, psd, back,
                           df_a2z, bounds, low_bound_freq=1500, up_bound_freq=5000,
                           param_wdw=None, nsteps=1000, filename=None, parallelise=False,
                           progress=False, nwalkers=64, normalise=False, degrees=None, instr='kepler',
-                          use_sinc=False, asym_profile='korzennik') :
+                          use_sinc=False, asym_profile='korzennik', save_only_after_sampling=False,
+                          fit_amp=False, projected_splittings=False, existing_chains='read') :
 
   '''
   Use a MCMC to explore the distribution around the maximum likelihood estimation results. 
@@ -198,6 +212,9 @@ def explore_distribution (result, param_type, freq, psd, back,
   :rtype: emcee.EnsembleSampler
   '''
  
+  if save_only_after_sampling and existing_chains=='sample' :
+    raise Exception ("save_only_after_sampling=True and existing_chains='sample' are incompatible options.")
+ 
   psd_to_fit = psd / back
 
   aux_freq = freq[(freq>low_bound_freq)&(freq<up_bound_freq)]
@@ -216,7 +233,7 @@ def explore_distribution (result, param_type, freq, psd, back,
   # adding free constant for background
   result = np.append (result, 1.)
   lower_bounds = np.append (lower_bounds, 1e-6)
-  upper_bounds = np.append (upper_bounds, 3.)
+  upper_bounds = np.append (upper_bounds, 5.)
   param_type = np.append (param_type, 'background')
   degrees = np.append (degrees, 'a')
 
@@ -236,9 +253,29 @@ def explore_distribution (result, param_type, freq, psd, back,
 
   if filename is not None :
     if path.exists (filename) :
-      return None, None #this way, no sampling is performed and the existing file is left untouched
-    backend = emcee.backends.HDFBackend(filename)
-    backend.reset(nwalkers, ndim) #not sure this necessary with the previous check 
+      if existing_chains=='read' :
+        print (filename + " already exists, existing chains set to 'read', no sampling has been performed, proceeding to next step.")
+        sampler = emcee.backends.HDFBackend(filename, read_only=True)
+        return sampler, norm
+      elif existing_chains=='ignore' :
+        print (filename + " already exists, existing chains set to 'ignore', proceeding to next step.")
+        return None, None #this way, no sampling is performed and the existing file is left untouched
+      elif existing_chains=='reset' :
+        os.remove (filename)
+        backend = emcee.backends.HDFBackend(filename)
+        backend.reset(nwalkers, ndim)  
+        print (filename + " already exists, existing chains set to 'reset', file has been deleted and a new file has been created instead.")
+      elif existing_chains=='sample' :
+        backend = emcee.backends.HDFBackend(filename)
+        pos = None
+        print (filename + " already exists, existing chains set to 'sample', sampling will restart from where it stopped.")
+    else :
+      backend = emcee.backends.HDFBackend(filename)
+      backend.reset(nwalkers, ndim)  
+    if save_only_after_sampling :
+      # I have deliberately created the file to signal to other process that this chain is being
+      # sampled
+      backend = None
     #saving parameters name and normalisation information
     filename_info = filename[:len(filename)-3] + '.dat'
     if degrees is not None :
@@ -256,13 +293,18 @@ def explore_distribution (result, param_type, freq, psd, back,
   sampler = emcee.EnsembleSampler(nwalkers, ndim, log_probability, 
                                   args=(param_type, aux_freq, aux_psd, aux_back, 
                                         df_a2z, transform, bounds, param_wdw, norm, instr,
-                                        use_sinc, asym_profile),
+                                        use_sinc, asym_profile, fit_amp, projected_splittings),
                                   backend=backend, pool=pool)
   sampler.run_mcmc(pos, nsteps, progress=progress)
 
+  if filename is not None :
+    if save_only_after_sampling :
+      save_sampled_chain (filename, sampler, ndim, nwalkers, nsteps)
+
   return sampler, norm 
 
-def show_chain (filename, thin=100, read_info=False, save=False, discard=0) :
+def show_chain (filename, thin=100, read_info=False, save=False, discard=0,
+                fit_amp=False, bins=100) :
   '''
   Framework to plot chains saved as hdf5 files.
 
@@ -270,6 +312,9 @@ def show_chain (filename, thin=100, read_info=False, save=False, discard=0) :
   'mcmc_sampler_order_[n_order]_degrees_[n_degrees].h5' with n_degrees being '13' or '02'.
   :type filename: str
 
+  :param bins: number of bins for each cornerplot panel. Optional, default 100.
+  :type bins: int
+
   :return: None
   '''
 
@@ -277,6 +322,10 @@ def show_chain (filename, thin=100, read_info=False, save=False, discard=0) :
   
   if read_info :
     labels = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=0)
+    if fit_amp :
+      labels[labels=='height'] = 'amplitude'
+    if projected_splittings :
+      labels[labels=='split'] = 'proj_split'
     degrees = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=2)
     labels = np.char.add (labels, np.full (labels.size, '_', dtype=str))
     labels = np.char.add (labels, degrees)
@@ -290,9 +339,9 @@ def show_chain (filename, thin=100, read_info=False, save=False, discard=0) :
   if read_info :
     flatchain = flatchain * norm
     labels, degrees, flatchain = sort_chain (labels, degrees, flatchain)
-    fig = corner.corner(flatchain, labels=labels, quantiles=quantiles, show_titles=True, title_fmt='.4f', bins=100)
+    fig = corner.corner(flatchain, labels=labels, quantiles=quantiles, show_titles=True, title_fmt='.4f', bins=bins)
   else :
-    fig = corner.corner(flatchain, labels=labels, quantiles=quantiles, show_titles=True, title_fmt='.4f', bins=100)
+    fig = corner.corner(flatchain, labels=labels, quantiles=quantiles, show_titles=True, title_fmt='.4f', bins=bins)
   fig.set_size_inches(24,24)
 
   if save :
@@ -304,7 +353,8 @@ def show_chain (filename, thin=100, read_info=False, save=False, discard=0) :
 def wrap_explore_distribution (df_a2z, freq, psd, back, low_bound_freq=1500, up_bound_freq=5000, wdw=None,
                                nsteps=1000, show_corner=False, filename=None, parallelise=False, progress=False,
                                nwalkers=64, normalise=False, instr='kepler', coeff_discard=50, use_sinc=False,
-                               asym_profile='korzennik') :
+                               asym_profile='korzennik', save_only_after_sampling=False, fit_amp=False,
+                               projected_splittings=False, bins=100, existing_chains='read') :
 
   '''
   A small wrapper to call and exploit restult of the explore_distribution function inside the peakbagging
@@ -343,13 +393,16 @@ def wrap_explore_distribution (df_a2z, freq, psd, back, low_bound_freq=1500, up_
     or Nigam & Kosovichev 1998 (``nigam-kosovichev``). 
   :type asym_profile: str
 
+  :param bins: number of bins for each cornerplot panel. Optional, default 100.
+  :type bins: int
+
   :return: the 16, 50 and 84th percentiles values for each parameter 
   :rtype: ndarray
   '''
 
   df_to_pass = df_a2z.copy ()
 
-  result, param_type, bounds, degrees = a2z_df_to_param (df_a2z, give_degree=True)
+  result, param_type, bounds, degrees = a2z_df_to_param (df_a2z, give_degree=True, fit_amp=fit_amp)
 
   param_wdw = None
   if wdw is not None :
@@ -363,7 +416,9 @@ def wrap_explore_distribution (df_a2z, freq, psd, back, low_bound_freq=1500, up_
                             df_to_pass, cp_bounds, low_bound_freq=low_bound_freq, filename=filename, 
                             up_bound_freq=up_bound_freq, param_wdw=param_wdw, nsteps=nsteps, 
                             parallelise=parallelise, progress=progress, nwalkers=nwalkers, normalise=normalise,
-                            degrees=degrees, instr=instr, use_sinc=use_sinc, asym_profile=asym_profile)
+                            degrees=degrees, instr=instr, use_sinc=use_sinc, asym_profile=asym_profile,
+                            save_only_after_sampling=save_only_after_sampling, fit_amp=fit_amp, 
+                            projected_splittings=projected_splittings, existing_chains=existing_chains)
   if sampler is None :
     return None #sampler is None mean the hdf5 file already existed and no sampling has been performed
 
@@ -373,11 +428,13 @@ def wrap_explore_distribution (df_a2z, freq, psd, back, low_bound_freq=1500, up_
   thin = 1
 
   flat_samples = sampler.get_chain(discard=discard, thin=thin, flat=True)
-  labels = np.append (param_type, 'background')
-  truths = np.append (result, 1.)
 
   if show_corner :
     labels = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=0)
+    if fit_amp :
+      labels[labels=='height'] = 'amplitude'
+    if projected_splittings :
+      labels[labels=='split'] = 'proj_split'
     degrees = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=2)
     labels = np.char.add (labels, np.full (labels.size, '_', dtype=str))
     labels = np.char.add (labels, degrees)
@@ -387,7 +444,7 @@ def wrap_explore_distribution (df_a2z, freq, psd, back, low_bound_freq=1500, up_
     flat_samples = flat_samples * norm
     labels, degrees, flatchain = sort_chain (labels, degrees, flat_samples)
 
-    fig = corner.corner(flatchain, labels=labels, quantiles=quantiles, show_titles=True, title_fmt='.4f', bins=100)
+    fig = corner.corner(flatchain, labels=labels, quantiles=quantiles, show_titles=True, title_fmt='.4f', bins=bins)
     fig.set_size_inches(24,24)
     if filename is not None :
       plt.savefig (filename[:len(filename)-3]+'.pdf', format='pdf')
@@ -397,59 +454,3 @@ def wrap_explore_distribution (df_a2z, freq, psd, back, low_bound_freq=1500, up_
 
   return centiles
 
-def chain_to_a2z (filename, thin=1, discard=500) :
-  '''
-  Function to create an a2z dataframe from a sampled chain.
-
-  :param filename: name of the chain. The name format is the following
-  'mcmc_sampler_order_[n_order]_degrees_[n_degrees].h5' with n_degrees being '13' or '02'.
-  :type filename: str
-
-  :return: a2z style dataframe.
-  :rtype: pandas DataFrame
-  '''
-
-  reader = emcee.backends.HDFBackend(filename, read_only=True)
-
-  order = filename[19:21]
-  order = int (order)
-  
-  labels = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=0)
-  degrees = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=2)
-  norm = np.loadtxt (filename[:len(filename)-3]+'.dat', usecols=1)
-
-  flatchain = reader.get_chain(flat=True, thin=thin)
-  centiles = np.percentile (flatchain, [16, 50, 84], axis=0) * norm
-  bounds = np.percentile (flatchain, [0,100], axis=0) * norm
-
-  columns = list (range (9))
-  aux_index = list (range (labels.size))
-
-  df = pd.DataFrame (index=aux_index, columns=columns)
-  df.loc[:,0] = order
-  df.loc[:,1] = degrees
-  df.loc[:,2] = labels
-  df.loc[labels!='a',3] = 'mode'
-  df.loc[labels=='a',3] = 'order'
-  df.loc[:,4] = centiles[1,:]
-
-  #retransforming width and heights
-  cond_exp = (df[2]=='height')|(df[2]=='width')
-  a_cond_exp = cond_exp.to_numpy ()
-  for ii in range (centiles.shape[0]) :
-    centiles[ii,a_cond_exp] = np.exp (centiles[ii,a_cond_exp])
-  df.loc[cond_exp,4] = np.exp (df.loc[cond_exp,4])
-
-  #Computing (symmetric) sigmas
-  sigma_1 = centiles[1,:] - centiles[0,:]
-  sigma_2 = centiles[2,:] - centiles[1,:]
-  sigma = np.maximum (sigma_1, sigma_2)
-
-  df.loc[:,5] = sigma
-  df.loc[:,6] = 0.
-
-  #Adding bounds
-  df.loc[:,7] = bounds[0,:]
-  df.loc[:,8] = bounds[1,:]
-
-  return df
