@@ -25,10 +25,10 @@ def dnu_from_df (df_a2z) :
 
 def restrict_input (df) :
 
-  '''
-  Restrict input that is used to fit the model. Can increase computation performances
-  and reduce the asymmetry wing effect when considering Nigam-Kosovichev implementation of 
-  asymmetry. 
+  ''' 
+  Restrict input that is used to fit the model. Can increase computation
+  performances and reduce the asymmetry wing effect when considering
+  Nigam-Kosovichev implementation of asymmetry.  
   '''
 
   cond = (df[6]==0)|(df[0]=='a')
@@ -179,19 +179,22 @@ def get_list_order (df_a2z) :
 
   return order
 
-def peakbagging (a2z_file, freq, psd, back=None, wdw=None, dnu=None, spectro=True,
-                 nsteps_mcmc=1000, show_corner=False,
-                 store_chains=False, mcmcDir='.', order_to_fit=None, parallelise=False, progress=False,
-                 strategy='order', fit_02=True, fit_13=True, nwalkers=64, normalise=False, instr='geometric', show_summary=False,
-                 filename_summary=None, show_prior=False, coeff_discard=50, use_sinc=False, asym_profile='nigam-kosovichev',
-                 restrict=False, remove_asymmetry_outside_window=True, do_not_use_dnu=False, save_only_after_sampling=False,
-                 size_window=None, fit_amp=False, extended=False, projected_splittings=False, bins=100, existing_chains='read',
-                 fit_splittings=True, fit_angle=True) :
+def peakbagging (a2z_file, freq, psd, back=None, wdw=None, dnu=None,
+                 spectro=True, nsteps_mcmc=1000, show_corner=False, store_chains=False,
+                 mcmcDir='.', order_to_fit=None, parallelise=False, progress=False,
+                 strategy='order', fit_02=True, fit_13=True, nwalkers=64, normalise=False,
+                 instr='geometric', show_summary=False, filename_summary=None, show_prior=False,
+                 coeff_discard=50, use_sinc=False, asym_profile='nigam-kosovichev',
+                 restrict=False, remove_asymmetry_outside_window=True, do_not_use_dnu=False,
+                 save_only_after_sampling=False, size_window=None, fit_amp=False,
+                 extended=False, projected_splittings=False, bins=100, existing_chains='read',
+                 fit_splittings=True, fit_angle=True, fit_amplitude_ratio=False) : 
   '''
-  Read an a2z input file with a prior guess of the modes to fit and return the result 
-  of the peakbagging protocol processed over those modes. 
-  Fitting strategy is the following: pair 1-3 and 2-0 of same order are successively fitted from low frequencies
-  to high frequencies following a pseudo-global strategy. This protocol is repeted n_process times. 
+  Read an a2z input file with a prior guess of the modes to fit and return the
+  result of the peakbagging protocol processed over those modes.  Fitting
+  strategy is the following: pair 1-3 and 2-0 of same order are successively
+  fitted from low frequencies to high frequencies following a pseudo-global
+  strategy. This protocol is repeted n_process times. 
 
   :param a2z_file: name of the file to read the parameters.
   :type a2z_file: string
@@ -202,121 +205,152 @@ def peakbagging (a2z_file, freq, psd, back=None, wdw=None, dnu=None, spectro=Tru
   :param psd: real power vector
   :type psd: ndarray
 
-  :param back: activity background vector that will be used to complete the model to fit. Optional default ``None``.
-    Must have the same length than freq and psd. 
+  :param back: activity background vector that will be used to complete the
+    model to fit. Optional default ``None``.  Must have the same length than freq
+    and psd. 
+
   :type back: ndarray.
 
-  :param wdw: observation window (0 and 1 array of the same lenght as the original timeseries)
-    to analyse in order to predict the sidelob pattern. Optional, default ``None``. 
+  :param wdw: observation window (0 and 1 array of the same lenght as the
+    original timeseries) to analyse in order to predict the sidelob pattern.
+    Optional, default ``None``. 
   :type wdw: ndarray.
 
-  :param dnu: large separation of the modes. In this function, it will only be used to adapt the fitting window for 
-    fitted modes above 1200 muHz. If not given and if at least two l=0 modes are specified in the a2z file, dnu will be automatically computed. 
+  :param dnu: large separation of the modes. In this function, it will only be
+    used to adapt the fitting window for fitted modes above 1200 muHz. If not given
+    and if at least two l=0 modes are specified in the a2z file, dnu will be
+    automatically computed. 
   :type dnu: float.
 
   :param nsteps_mcmc: number of steps to process into each MCMC exploration.
   :type nsteps_mcmc: int
 
-  :param show_corner: if set to ``True``, show and save the corner plot for the posterior distribution sampling.
+  :param show_corner: if set to ``True``, show and save the corner plot for the
+    posterior distribution sampling.
   :type show_corner: bool
 
-  :param store_chains: if set to ``True``, each MCMC sampler will be stored as an hdf5 files. Filename will be autogenerated
-    with modes parameters. Optional, default ``False``
+  :param store_chains: if set to ``True``, each MCMC sampler will be stored as
+    an hdf5 files. Filename will be autogenerated with modes parameters. Optional,
+    default ``False``
   :type store_chains: bool
 
   :param mcmcDir: directory where to save the MCMC sampler. Optional, default ``.``.
   :type mcmcDir: string
  
-  :param order_to_fit: list of order to fit if the input a2z file contains order that are supposed not to be fitted.
-    Optional, default ``None``.
+  :param order_to_fit: list of order to fit if the input a2z file contains
+    order that are supposed not to be fitted.  Optional, default ``None``.
   :type order_to_fit: array-like
 
-  :param parallelise: If set to ``True``, use Python multiprocessing tool to parallelise process.
-    Optional, default ``False``.
+  :param parallelise: If set to ``True``, use Python multiprocessing tool to
+    parallelise process.  Optional, default ``False``.
   :type parallelise: bool
 
-  :param strategy: strategy to use for the fit, ``order``, ``pair``, ``global``. Optional, default ``order``. If 'pair' is used, a2z input must contain
-    individual heights, widths and splittings for each degree. The ``global`` behaviour is experimental. 
+  :param strategy: strategy to use for the fit, ``order``, ``pair``,
+    ``global``. Optional, default ``order``. If 'pair' is used, a2z input must
+    contain individual heights, widths and splittings for each degree. The
+    ``global`` behaviour is experimental. 
   :type strategy: str
 
-  :param fit_02: if strategy is set to ``pair``, l=0 and 2 modes will only be fitted if this parameter is set to ``True``.
-    Optional, default ``True``.
+  :param fit_02: if strategy is set to ``pair``, l=0 and 2 modes will only be
+    fitted if this parameter is set to ``True``.  Optional, default ``True``.
   :type fit_02: bool
 
-  :param fit_13: if strategy is set to ``pair``, l=1 and 3 modes will only be fitted if this parameter is set to ``True``.
-    Optional, default ``True``.
+  :param fit_13: if strategy is set to ``pair``, l=1 and 3 modes will only be
+    fitted if this parameter is set to ``True``.  Optional, default ``True``.
   :type fit_13: bool
 
   :param nwalkers: number of wlakers in the MMCM process.
   :type nwalkers: int
 
-  :param instr: instrument to consider (amplitude ratio inside degrees depend on geometry 
-    AND instrument and should be adaptated). Possible argument : ``geometric``, ``kepler``, ``golf``, ``virgo``.
-    Optional, default ``geometric``. 
+  :param instr: instrument to consider (amplitude ratio inside degrees depend
+    on geometry AND instrument and should be adaptated). Possible argument :
+    ``geometric``, ``kepler``, ``golf``, ``virgo``.  Optional, default
+    ``geometric``. 
   :type instr: str
 
-  :param show_summary: show summary plot at the end of the fit. Optional, default ``False``.
+  :param show_summary: show summary plot at the end of the fit. Optional,
+    default ``False``.
   :type show_summary: bool
 
-  :param coeff_discard: coeff used to compute the number of values to discard at the beginning
-    of each MCMC : total amount of sampled values will be divided by coeff_discard. Optional, default 50.
+  :param coeff_discard: coeff used to compute the number of values to discard
+    at the beginning of each MCMC : total amount of sampled values will be divided
+    by coeff_discard. Optional, default 50.
   :type coeff_discard: int
 
-  :param use_sinc: if set to ``True``, mode profiles will be computed using cardinal sinus and not Lorentzians.
-    No asymmetry term will be used if it is the case. Optional, default ``False``.
+  :param use_sinc: if set to ``True``, mode profiles will be computed using
+    cardinal sinus and not Lorentzians.  No asymmetry term will be used if it is
+    the case. Optional, default ``False``.
   :type use_sinc: bool
 
-  :param asym_profile: depending on the chosen argument, asymmetric profiles will be computed following Korzennik 2005 (``korzennik``)
-    or Nigam & Kosovichev 1998 (``nigam-kosovichev``). Default ``nigam-kosovichev``. 
+  :param asym_profile: depending on the chosen argument, asymmetric profiles
+    will be computed following Korzennik 2005 (``korzennik``) or Nigam & Kosovichev
+    1998 (``nigam-kosovichev``). Default ``nigam-kosovichev``. 
   :type asym_profile: str
 
-  :param restrict: if set to True, will only use the modes with unfrozen parameters in the model.
+  :param restrict: if set to True, will only use the modes with unfrozen
+    parameters in the model.
   :type restrict: bool
 
-  :param remove_asymmetry_outside_window: if set to True, asymmetry of modes outside of the fitting window will be set to 0. 
-    Optional, default ``True``. 
+  :param remove_asymmetry_outside_window: if set to True, asymmetry of modes
+    outside of the fitting window will be set to 0.  Optional, default ``True``. 
   :type remove_asymmetry_outside_window: bool 
 
-  :param do_not_use_dnu: if set to ``True``, fitting window will be computed without using dnu value.
+  :param do_not_use_dnu: if set to ``True``, fitting window will be computed
+    without using dnu value.
   :type do_not_use_dnu: bool
 
-  :param save_only_after_sampling: if set to True, hdf5 file with chains information will only be saved at the end of the sampling
-    process. If set to False, the file will be saved step by step (see ``emcee`` documentation).
+  :param save_only_after_sampling: if set to True, hdf5 file with chains
+    information will only be saved at the end of the sampling process. If set to
+    False, the file will be saved step by step (see ``emcee`` documentation).
   :type saveon_only_after_sampling: bool
 
-  :param size_window: size of the fitting window, in muHz. If not given, the size of the fitting window will be automatically set,
-    using dnu or the input frequencies of the parameter to fit. Optional, default ``None``.
+  :param size_window: size of the fitting window, in muHz. If not given, the
+    size of the fitting window will be automatically set, using dnu or the input
+    frequencies of the parameter to fit. Optional, default ``None``.
   :type size_window: float
 
-  :param fit_amp: if set to ``True``, amplitude of the modes will be fitted instead of heights (input heights will be transformed inside
-    the code and retransformed as outputs).
+  :param fit_amp: if set to ``True``, amplitude of the modes will be fitted
+    instead of heights (input heights will be transformed inside the code and
+    retransformed as outputs).
   :type fit_amp: bool
 
-  :param extended: if set to ``True``, the returned pkb array will be extended to contain asymmetric uncertainties over each parameter.
+  :param extended: if set to ``True``, the returned pkb array will be extended
+    to contain asymmetric uncertainties over each parameter.
   :type extended: bool
 
-  :param projected_splittings: if set to ``True``, the function will consider that the ``split`` parameters of the input are projected 
-    splittings and will build the model consequently. Optional, default ``False``. 
+  :param projected_splittings: if set to ``True``, the function will consider
+    that the ``split`` parameters of the input are projected splittings and will
+    build the model consequently. Optional, default ``False``. 
   :type projected_splittings: bool
 
   :param bins: number of bins for each cornerplot panel. Optional, default 100.
   :type bins: int
 
-  :param existing_chains: controls the behaviour of the function concerning existing hdf5 files. If, ``read``, existing files will be read
-    without sampling and function output will be updated consequently, if ``reset``, the backend will be cleared and the chain will 
-    be sampled from scratch, if ``sample`` the function will sample the chain from where the previous exploration was stopped,
+  :param existing_chains: controls the behaviour of the function concerning
+    existing hdf5 files. If, ``read``, existing files will be read without sampling
+    and function output will be updated consequently, if ``reset``, the backend
+    will be cleared and the chain will be sampled from scratch, if ``sample`` the
+    function will sample the chain from where the previous exploration was stopped,
     if ``ignore``, the chain is not read. Optional, default ``read``.
   :type existing_chains: str
 
-  :param fit_splittings: if the selected strategy is ``global``, the global ``split`` parameter will be fitted only
-    if this parameter is set to ``True``. Optional, default ``True``. 
+  :param fit_splittings: if the selected strategy is ``global``, the global
+    ``split`` parameter will be fitted only if this parameter is set to ``True``.
+     Optional, default ``True``. 
   :type fit_splittings: bool
 
-  :param fit_angle: if the selected strategy is ``global``, the global ``angle`` parameter will be fitted only 
-    if this parameter is set to ``True``. Optional, default ``True``. 
+  :param fit_angle: if the selected strategy is ``global``, the global
+    ``angle`` parameter will be fitted only if this parameter is set to ``True``.
+    Optional, default ``True``. 
   :type fit_angle: bool
 
-  :return: a2z fitted modes parameters as a DataFrame, and corresponding pkb array (extended pkb array if extended is set to ``True``)
+  :param fit_amplitude_ratio: if the selected strategy is ``global``, the global
+    ``amp_l`` amplitude ratio between degrees parameters will be fitted only if
+    this parameter is set to ``True``.  Optional, default ``True``. 
+  :type fit_amplitude_ratio: bool
+
+  :return: a2z fitted modes parameters as a DataFrame, and corresponding pkb
+    array (extended pkb array if extended is set to ``True``)
   :rtype: tuple of objects
   '''
 
@@ -341,11 +375,13 @@ def peakbagging (a2z_file, freq, psd, back=None, wdw=None, dnu=None, spectro=Tru
 
   if strategy=='global' :
     print ('Global fitting')
-    df_a2z.loc[df_a2z[2]!='amp_l', 6] = 0
+    df_a2z.loc[:, 6] = 0
     if not fit_splittings :
       df_a2z.loc[(df_a2z[2]=='split')&(df_a2z[0]=='a'), 6] = 1
     if not fit_angle :
       df_a2z.loc[(df_a2z[2]=='angle')&(df_a2z[0]=='a'), 6] = 1
+    if not fit_amplitude_ratio :
+      df_a2z.loc[(df_a2z[2]=='amp_l')&(df_a2z[0]=='a'), 6] = 1
 
     up_bound, low_bound = define_window (df_a2z, strategy, dnu=dnu, 
                                          do_not_use_dnu=do_not_use_dnu,
