@@ -31,10 +31,12 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
                        reboxing_behaviour='advanced_reboxing', reboxing_strategy='median',
                        save_only_after_sampling=False, apodisation=False, fit_angle=False,
                        guess_angle=90, fit_splittings=False, guess_split=0,
-                       frozen_harvey_exponent=False, fit_amp=False, extended=False, author=None,
+                       fit_amplitude_ratio=False, frozen_harvey_exponent=False, 
+                       fit_amp=False, extended=False, author=None,
                        pkb_fmt=None, projected_splittings=False, bins=100,
                        existing_chains_back='read', existing_chains_pattern='read',
-                       existing_chains_pkb='read', strategy='order', common_width=False) :
+                       existing_chains_pkb='read', strategy='order', common_width=False,
+                       save_background_vector=True) :
 
   '''
   Framework for stellar peakbagging considering only a few input parameters.
@@ -58,16 +60,20 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
   :param nwalkers: number of walkers for the MCMC process. Optional, default 64.
   :type nwalkers: int
 
-  :param dnu: large frequency separation. Must be given in µHz. If ``None``, will be computed through scaling laws. 
+  :param dnu: large frequency separation. Must be given in µHz. 
+    If ``None``, will be computed through scaling laws. 
     Optional, default ``None``.
   :type dnu: float
 
-  :param back: array of fitted background, of same length as *freq* and *psd*. If given, no fit will be performed on background
-    and the pattern step will directly take place. ``Hmax``, ``Wenv`` and ``numax`` must in this case also be given. 
+  :param back: array of fitted background, of same length as ``freq`` 
+    and ``psd``. If given, no fit will be performed on background
+    and the pattern step will directly take place. ``Hmax``, ``Wenv`` 
+    and ``numax`` must in this case also be given. 
     Optional, default ``None``.
   :type back: ndarray
 
-  :param numax: maximum of power in p-mode envelope. Must be given in µHz. Optional, default ``None``.
+  :param numax: maximum of power in p-mode envelope. Must be given in µHz. 
+    Optional, default ``None``.
   :type numax: float
 
   :param Hmax: amplitude of p-mode envelope. Optional, default ``None``.
@@ -106,19 +112,20 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
     computed by the automatic guess. Optional, default ``None``. 
   :type d13: float
 
-  :param n_harvey: number of Harvey laws to use to build the background
-    model. Optional, default 2. With more than two Harvey laws, it is strongly recommended
+  :param n_harvey: number of Harvey laws to use to build the background model.
+    Optional, default 2. With more than two Harvey laws, it is strongly recommended
     to manually feed the 'guess' parameter.  
   :type n_harvey: int
 
-  :param guess_back: first guess directly passed by the user. If guess is ``None``, the 
-    function will automatically infer a first guess. Optional, default ``None``.
-    Backgrounds parameters given in the following order:
-    *param_harvey (3* ``n_harvey`` *), param_plaw (2), param_gaussian (3), white noise (1)*. 
+  :param guess_back: first guess directly passed by the user. If guess is
+    ``None``, the function will automatically infer a first guess. Optional,
+    default ``None``.  Backgrounds parameters given in the following order:
+    *param_harvey (3* ``n_harvey`` *), param_plaw (2), param_gaussian (3), white
+    noise (1)*. 
   :type guess_back: array-like.
 
-  :param spectro: if set to ``True``, make the plot with unit consistent with radial velocity, else with 
-    photometry. Optional, default ``False``.
+  :param spectro: if set to ``True``, make the plot with unit consistent with
+    radial velocity, else with photometry. Optional, default ``False``.
   :type spectro: bool
 
   :param filename_back: name of the of the dat file of the fitted background, 
@@ -146,16 +153,18 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
     be performed for the pattern. Optional, default ``True``.
   :type pattern_mcmc: bool
   
-  :param n_order: number of orders to consider for the global pattern fit. Optional, default 3. 
+  :param n_order: number of orders to consider for the global pattern fit.
+    Optional, default 3. 
   :type n_order: int
 
   :param n_order_peakbagging: number of orders to fit at the 
     individual mode parameters step. Optional, default 3. 
   :type n_order_peakbagging: int
 
-  :param h0_screening: if set to ``True``, a fast frequency screening of the p-mode region 
-    will be performed in order to determine the mode that it possible to fit. ``n_order_peakbagging``
-    value will be ignored in this case. Optional, default ``False``.
+  :param h0_screening: if set to ``True``, a fast frequency screening of the
+    p-mode region will be performed in order to determine the mode that it possible
+    to fit. ``n_order_peakbagging`` value will be ignored in this case. Optional,
+    default ``False``.
   :type h0_screening: bool
 
   :param filename_pattern: name of the pdf file where the fitted global pattern parameters 
@@ -221,104 +230,136 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
     Optional, default ``summary_peakbagging.pdf``.
   :type filename_peakbagging: str
 
-  :param nopeakbagging: if set to ``True``, individual modes parameters will not be fitted. Optional, default ``False``.
+  :param nopeakbagging: if set to ``True``, individual modes parameters will
+    not be fitted. Optional, default ``False``.  
   :type nopeakbagging: bool
 
-  :param coeff_discard_back: coefficient used to compute the number of values to discard at the beginning
-    of each MCMC sampled by the background step : total amount of sampled values will be divided by coeff_discard. Optional, default 50.
+  :param coeff_discard_back: coefficient used to compute the number of values
+    to discard at the beginning of each MCMC sampled by the background step : total
+    amount of sampled values will be divided by coeff_discard. Optional, default
+    50.
   :type coeff_discard: int
 
-  :param coeff_discard_pattern: coefficient used to compute the number of values to discard at the beginning
-    of each MCMC sampled by the pattern step : total amount of sampled values will be divided by coeff_discard. Optional, default 50.
+  :param coeff_discard_pattern: coefficient used to compute the number of
+    values to discard at the beginning of each MCMC sampled by the pattern step :
+    total amount of sampled values will be divided by coeff_discard. Optional,
+    default 50.
   :type coeff_discard: int
 
-  :param coeff_discard_pkb: coefficient used to compute the number of values to discard at the beginning
-    of each MCMC sampled by the peakbagging step : total amount of sampled values will be divided by coeff_discard. Optional, default 50.
+  :param coeff_discard_pkb: coefficient used to compute the number of values to
+    discard at the beginning of each MCMC sampled by the peakbagging step : total
+    amount of sampled values will be divided by coeff_discard. Optional, default
+    50.
   :type coeff_discard: int
 
-  :param thin_back: take only every ``thin`` steps from the chain sampled for backgrounds parameters. Optional, default 10. 
+  :param thin_back: take only every ``thin`` steps from the chain sampled for
+    backgrounds parameters. Optional, default 10. 
   :type thin_back: int
 
-  :param thin_pattern: take only every ``thin`` steps from the chain sampled for pattern parameters. Optional, default 10. 
+  :param thin_pattern: take only every ``thin`` steps from the chain sampled
+    for pattern parameters. Optional, default 10. 
   :type thin_pattern: int
 
-  :param quickfit: if set to ``True``, the fit will be performed over a smoothed and logarithmically resampled background.
-    Optional, default ``False``. 
+  :param quickfit: if set to ``True``, the fit will be performed over a
+    smoothed and logarithmically resampled background.  Optional, default
+    ``False``. 
   :type quickfit: bool
 
-  :param num: number of point to consider in the degraded PSD if ``quickfit`` is set to True. Optional, default 500.
+  :param num: number of point to consider in the degraded PSD if ``quickfit``
+    is set to True. Optional, default 500.
   :type num: int
 
-  :param reboxing_behaviour: behaviour for ``quickfit`` new power vector computation. It can be ``smoothing``, ``reboxing`` or 
-    ``advanced_reboxing``. Optional, default ``advanced_reboxing``. 
+  :param reboxing_behaviour: behaviour for ``quickfit`` new power vector
+    computation. It can be ``smoothing``, ``reboxing`` or ``advanced_reboxing``.
+    Optional, default ``advanced_reboxing``. 
   :type reboxing_behaviour: str
 
-  :param reboxing_strategy: reboxing strategy to take box values when using ``quickfit`` and ``reboxing_behaviour=reboxing``. 
-    Can be ``median`` or ``mean``. Optional, default ``median``.
+  :param reboxing_strategy: reboxing strategy to take box values when using
+    ``quickfit`` and ``reboxing_behaviour=reboxing``.  Can be ``median`` or
+    ``mean``. Optional, default ``median``.
   :type reboxing_strategy: str
 
-  :param save_only_after_sampling: if set to True, hdf5 file with chains information will only be saved at the end of the sampling
-    process. If set to False, the file will be saved step by step (see ``emcee`` documentation).
+  :param save_only_after_sampling: if set to True, hdf5 file with chains
+    information will only be saved at the end of the sampling process. If set to
+    False, the file will be saved step by step (see ``emcee`` documentation).
   :type saveon_only_after_sampling: bool
 
-  :param apodisation: if set to ``True``, distort the model to take the apodisation into account. Optional, default ``False``.
+  :param apodisation: if set to ``True``, distort the model to take the
+    apodisation into account. Optional, default ``False``.
   :type apodisation: bool
 
-  :param fit_angle: if set to ``True``, the peakbagging process will include inclination angle of the star in the parameters to fit.
-    Optional, default ``False``.
+  :param fit_angle: if set to ``True``, the peakbagging process will include
+    inclination angle of the star in the parameters to fit.  Optional, default
+    ``False``.
   :type fit_angle: bool
 
-  :param guess_angle: initial guess value for angle. Will be taken into account only if ``fit_angle`` is ``True``. 
-    Optional, default 90.
+  :param guess_angle: initial guess value for angle. Will be taken into account
+    only if ``fit_angle`` is ``True``.  Optional, default 90.
   :type guess_angle: float
 
-  :param fit_splittings: if set to ``True``, the peakbagging process will include mode splittings in the parameters to fit.
-    Optional, default ``False``.
+  :param fit_splittings: if set to ``True``, the peakbagging process will
+    include mode splittings in the parameters to fit.  Optional, default ``False``.
   :type fit_splittings: bool
 
-  :param guess_split: initial guess value for splittings. Will be taken into account only if ``fit_splittings`` is ``True``. 
-    Optional, default 0.
+  :param guess_split: initial guess value for splittings. Will be taken into
+    account only if ``fit_splittings`` is ``True``.  Optional, default 0.
   :type guess_split: float
 
-  :param frozen_harvey_exponent: set to True to have the Harvey models exponent fixed to 4. Optional, default ``False``.
+  :param fit_amplitude_ratio: if the selected strategy is ``global``, the global
+    ``amp_l`` amplitude ratio between degrees parameters will be fitted only if
+    this parameter is set to ``True``.  Optional, default ``True``. 
+  :type fit_amplitude_ratio: bool
+
+  :param frozen_harvey_exponent: set to True to have the Harvey models 
+    exponent fixed to 4. Optional, default ``False``.
   :type frozen_harvey_exponent: bool
 
-  :param fit_amp: if set to ``True``, amplitude of the modes will be fitted instead of heights (input heights will be transformed inside
+  :param fit_amp: if set to ``True``, amplitude of the modes will be fitted 
+    instead of heights (input heights will be transformed inside
     the code and retransformed as outputs).
   :type fit_amp: bool
 
-  :param extended: if set to ``True``, the returned pkb array will be extended to contain asymmetric uncertainties over each parameter.
+  :param extended: if set to ``True``, the returned pkb array will be 
+    extended to contain asymmetric uncertainties over each parameter.
   :type extended: bool
 
-  :param author: identifier of the peakbagger, if given it will be specified in the header of the output pkb file. 
-    Optional, default ``None``.
+  :param author: identifier of the peakbagger, if given it will be specified 
+    in the header of the output pkb file. Optional, default ``None``.
   :type author: str
 
-  :param pkb_fmt: float formatting of the pkb file that will be created by the function. Optional, default ``None``.
+  :param pkb_fmt: float formatting of the pkb file that will be created by 
+    the function. Optional, default ``None``.
   :type fmt: str or array-like
 
-  :param projected_splittings: if set to ``True``, the function will consider that the ``split`` parameters of the input are projected 
+  :param projected_splittings: if set to ``True``, the function will consider 
+    that the ``split`` parameters of the input are projected 
     splittings and will build the model consequently. Optional, default ``False``. 
   :type projected_splittings: bool
 
   :param bins: number of bins for each cornerplot panel. Optional, default 100.
   :type bins: int
 
-  :param existing_chains_back: controls the behaviour of the function concerning a background parameters existing hdf5 file. 
-    If, ``read``, existing files will be read
-    without sampling and function output will be updated consequently, if ``reset``, the backend will be cleared and the chain will 
-    be sampled from scratch, if ``sample`` the function will sample the chain from where the previous exploration was stopped,
-    if ``ignore``, the chain is not read. Optional, default ``read``. The option ``ignore`` will apply only to individual mode chains,
-    it will be changed to ``read`` for background and pattern steps. 
+  :param existing_chains_back: controls the behaviour of the function
+    concerning a background parameters existing hdf5 file. If, ``read``, existing
+    files will be read without sampling and function output will be updated
+    consequently, if ``reset``, the backend will be cleared and the chain will be
+    sampled from scratch, if ``sample`` the function will sample the chain from
+    where the previous exploration was stopped, if ``ignore``, the chain is not
+    read. Optional, default ``read``.  The option ``ignore`` will apply only to
+    individual mode chains, it will be changed to ``read`` for background and
+    pattern steps. 
   :type existing_chains: str
 
-  :param existing_chains_pattern: Same as ``existing_chains_back`` but for a global pattern parameters existing hdf5 file. 
+  :param existing_chains_pattern: Same as ``existing_chains_back`` but for a 
+    global pattern parameters existing hdf5 file. 
   :type existing_chains_pattern: str
 
-  :param existing_chains_pkb: Same as ``existing_chains_back`` but for individual mode parameters existing hdf5 files. 
+  :param existing_chains_pkb: Same as ``existing_chains_back`` but for individual mode 
+    parameters existing hdf5 files. 
   :type existing_chains_pkb: str
 
-  :param strategy: fitting strategy for individual modes. Can be ``order`` or ``global``. Optional, default ``order``. Note that the ``global`` behaviour
+  :param strategy: fitting strategy for individual modes. Can be ``order`` or 
+    ``global``. Optional, default ``order``. Note that the ``global`` behaviour
     is experimental. 
   :type strategy: str
 
@@ -326,6 +367,10 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
     DataFrame. Optional, default ``False``.
   :type common_width: bool
 
+  :param save_background_vector: the full background vector will be saved only if 
+    this option is set to ``True``. Optional, default ``True``.
+  :type save_background_vector: bool
+
   :return: ``None``
   '''
 
@@ -368,11 +413,12 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
       numax = param_back[n_harvey*3+3]
     Wenv = param_back[n_harvey*3+4] 
 
-    np.savetxt (filename_back+'.dat', fitted_back, fmt='%-s', 
-                header=make_header_background_vector (author=author,apodisation=apodisation,
-                                                      low_cut=low_cut, nwalkers=nwalkers, nsteps=nsteps_mcmc_back, 
-                                                      coeff_discard=coeff_discard_back,
-                                                      reboxing_behaviour=reboxing_behaviour, quickfit=quickfit))
+    if save_background_vector :
+      np.savetxt (filename_back+'.dat', fitted_back, fmt='%-s', 
+                  header=make_header_background_vector (author=author,apodisation=apodisation,
+                                                        low_cut=low_cut, nwalkers=nwalkers, nsteps=nsteps_mcmc_back, 
+                                                        coeff_discard=coeff_discard_back,
+                                                        reboxing_behaviour=reboxing_behaviour, quickfit=quickfit))
 
   else :
     fitted_back = back
@@ -448,13 +494,17 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
     else :
       a2z_to_read = a2z_in
      
-    df_a2z, pkb = peakbagging (a2z_to_read, freq, psd, back=fitted_back, wdw=None, spectro=spectro,
-                   nsteps_mcmc=nsteps_mcmc_peakbagging, show_corner=True,
-                   store_chains=True, mcmcDir=mcmcDir, order_to_fit=None, parallelise=parallelise, progress=progress,
-                   strategy=strategy, nwalkers=nwalkers, normalise=False, instr=instr, filename_summary=filename_peakbagging, show_summary=False,
-                   coeff_discard=coeff_discard_pkb, save_only_after_sampling=save_only_after_sampling, restrict=True, fit_amp=fit_amp,
-                   extended=extended, projected_splittings=projected_splittings, bins=bins, existing_chains=existing_chains_pkb, 
-                   fit_splittings=fit_splittings, fit_angle=fit_angle)
+    df_a2z, pkb = peakbagging (a2z_to_read, freq, psd, back=fitted_back,
+                               wdw=None, spectro=spectro, nsteps_mcmc=nsteps_mcmc_peakbagging,
+                               show_corner=True, store_chains=True, mcmcDir=mcmcDir, order_to_fit=None,
+                               parallelise=parallelise, progress=progress, strategy=strategy,
+                               nwalkers=nwalkers, normalise=False, instr=instr,
+                               filename_summary=filename_peakbagging, show_summary=False,
+                               coeff_discard=coeff_discard_pkb,
+                               save_only_after_sampling=save_only_after_sampling, restrict=True,
+                               fit_amp=fit_amp, extended=extended, projected_splittings=projected_splittings,
+                               bins=bins, existing_chains=existing_chains_pkb, fit_splittings=fit_splittings,
+                               fit_angle=fit_angle, fit_amplitude_ratio=fit_amplitude_ratio)
 
     save_a2z (a2z_file, df_a2z)
 
